@@ -3,9 +3,7 @@ package DBM::Deep::Hash;
 use 5.006_000;
 
 use strict;
-use warnings;
-
-our $VERSION = q(1.0000);
+use warnings FATAL => 'all';
 
 use base 'DBM::Deep';
 
@@ -13,24 +11,9 @@ sub _get_self {
     eval { local $SIG{'__DIE__'}; tied( %{$_[0]} ) } || $_[0]
 }
 
-#XXX Need to add a check here for @_ % 2
-sub _repr { shift;return { @_ } }
-
-sub _import {
-    my $self = shift;
-    my ($struct) = @_;
-
-    foreach my $key (keys %$struct) {
-        $self->put($key, $struct->{$key});
-    }
-
-    return 1;
-}
+sub _repr { return {} }
 
 sub TIEHASH {
-    ##
-    # Tied hash constructor method, called by Perl's tie() function.
-    ##
     my $class = shift;
     my $args = $class->_get_args( @_ );
     
@@ -42,8 +25,8 @@ sub TIEHASH {
 sub FETCH {
     my $self = shift->_get_self;
     DBM::Deep->_throw_error( "Cannot use an undefined hash key." ) unless defined $_[0];
-    my $key = ($self->_storage->{filter_store_key})
-        ? $self->_storage->{filter_store_key}->($_[0])
+    my $key = ($self->_engine->storage->{filter_store_key})
+        ? $self->_engine->storage->{filter_store_key}->($_[0])
         : $_[0];
 
     return $self->SUPER::FETCH( $key, $_[0] );
@@ -52,8 +35,8 @@ sub FETCH {
 sub STORE {
     my $self = shift->_get_self;
     DBM::Deep->_throw_error( "Cannot use an undefined hash key." ) unless defined $_[0];
-	my $key = ($self->_storage->{filter_store_key})
-        ? $self->_storage->{filter_store_key}->($_[0])
+    my $key = ($self->_engine->storage->{filter_store_key})
+        ? $self->_engine->storage->{filter_store_key}->($_[0])
         : $_[0];
     my $value = $_[1];
 
@@ -63,8 +46,8 @@ sub STORE {
 sub EXISTS {
     my $self = shift->_get_self;
     DBM::Deep->_throw_error( "Cannot use an undefined hash key." ) unless defined $_[0];
-	my $key = ($self->_storage->{filter_store_key})
-        ? $self->_storage->{filter_store_key}->($_[0])
+    my $key = ($self->_engine->storage->{filter_store_key})
+        ? $self->_engine->storage->{filter_store_key}->($_[0])
         : $_[0];
 
     return $self->SUPER::EXISTS( $key );
@@ -73,62 +56,49 @@ sub EXISTS {
 sub DELETE {
     my $self = shift->_get_self;
     DBM::Deep->_throw_error( "Cannot use an undefined hash key." ) unless defined $_[0];
-	my $key = ($self->_storage->{filter_store_key})
-        ? $self->_storage->{filter_store_key}->($_[0])
+    my $key = ($self->_engine->storage->{filter_store_key})
+        ? $self->_engine->storage->{filter_store_key}->($_[0])
         : $_[0];
 
     return $self->SUPER::DELETE( $key, $_[0] );
 }
 
+# Locate and return first key (in no particular order)
 sub FIRSTKEY {
-	##
-	# Locate and return first key (in no particular order)
-	##
     my $self = shift->_get_self;
 
-	##
-	# Request shared lock for reading
-	##
-	$self->lock( $self->LOCK_SH );
-	
-	my $result = $self->_engine->get_next_key( $self );
-	
-	$self->unlock();
-	
-	return ($result && $self->_storage->{filter_fetch_key})
-        ? $self->_storage->{filter_fetch_key}->($result)
+    $self->lock_shared;
+    
+    my $result = $self->_engine->get_next_key( $self );
+    
+    $self->unlock;
+    
+    return ($result && $self->_engine->storage->{filter_fetch_key})
+        ? $self->_engine->storage->{filter_fetch_key}->($result)
         : $result;
 }
 
+# Return next key (in no particular order), given previous one
 sub NEXTKEY {
-	##
-	# Return next key (in no particular order), given previous one
-	##
     my $self = shift->_get_self;
 
-	my $prev_key = ($self->_storage->{filter_store_key})
-        ? $self->_storage->{filter_store_key}->($_[0])
+    my $prev_key = ($self->_engine->storage->{filter_store_key})
+        ? $self->_engine->storage->{filter_store_key}->($_[0])
         : $_[0];
 
-	##
-	# Request shared lock for reading
-	##
-	$self->lock( $self->LOCK_SH );
-	
-	my $result = $self->_engine->get_next_key( $self, $prev_key );
-	
-	$self->unlock();
-	
-	return ($result && $self->_storage->{filter_fetch_key})
-        ? $self->_storage->{filter_fetch_key}->($result)
+    $self->lock_shared;
+    
+    my $result = $self->_engine->get_next_key( $self, $prev_key );
+    
+    $self->unlock;
+    
+    return ($result && $self->_engine->storage->{filter_fetch_key})
+        ? $self->_engine->storage->{filter_fetch_key}->($result)
         : $result;
 }
 
-##
-# Public method aliases
-##
 sub first_key { (shift)->FIRSTKEY(@_) }
-sub next_key { (shift)->NEXTKEY(@_) }
+sub next_key  { (shift)->NEXTKEY(@_)  }
 
 sub _copy_node {
     my $self = shift;
